/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE or copy at https://www.boost.org/LICENSE_1_0.txt)
 **********************************************************************/

#pragma once

#include <memory>
#include <QObject>
#include <QDateTime>
#include <QHash>
#include <QColor>
#include <QCache>
#include <interfaces/azoth/ichatstyleresourcesource.h>

namespace LC
{
namespace Util
{
	class ResourceLoader;
}

namespace Azoth
{
class IMessage;
class ICLEntry;
class IAccount;
class IProxyObject;

namespace AdiumStyles
{
	class PackProxyModel;

	class AdiumStyleSource : public QObject
						   , public IChatStyleResourceSource
	{
		Q_OBJECT
		Q_INTERFACES (LC::Azoth::IChatStyleResourceSource)

		std::shared_ptr<Util::ResourceLoader> StylesLoader_;
		IProxyObject *Proxy_;

		PackProxyModel *PackProxyModel_;

		mutable QHash<QWebEnginePage*, QString> Frame2Pack_;
		mutable QHash<QString, QList<QColor>> Coloring2Colors_;
		mutable QString LastPack_;

		QHash<QObject*, QWebEnginePage*> Msg2Frame_;

		mutable QHash<QWebEnginePage*, QObject*> Frame2LastContact_;
	public:
		AdiumStyleSource (IProxyObject*, QObject* = 0);

		QAbstractItemModel* GetOptionsModel () const;
		QUrl GetBaseURL (const QString&) const;
		QString GetHTMLTemplate (const QString&,
				const QString&, QObject*, QWebEnginePage*) const;
		bool AppendMessage (QWebEnginePage*, QObject*, const ChatMsgAppendInfo&);
		void FrameFocused (QWebEnginePage*);
		QStringList GetVariantsForPack (const QString&);
	private:
		void PercentTemplate (QString&, const QMap<QString, QString>&) const;
		void SubstituteUserIcon (QString&,
				const QString&, bool, ICLEntry*, IAccount*);
		QString ParseMsgTemplate (QString templ, const QString& path,
				QWebEnginePage*, QObject*, const ChatMsgAppendInfo&);
	private slots:
		void handleMessageDelivered ();
		void handleMessageDestroyed ();
		void handleFrameDestroyed ();
	};
}
}
}
